@@ -1,23 +1,23 @@
 /*
- * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2003 Kungliga Tekniska Högskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -33,7 +33,7 @@
 
 #include "kx.h"
 
-RCSID("$Id: kxd.c 20452 2007-04-19 20:04:19Z lha $");
+RCSID("$Id$");
 
 static pid_t wait_on_pid = -1;
 static int   done        = 0;
@@ -48,7 +48,7 @@ childhandler (int sig)
      pid_t pid;
      int status;
 
-     do { 
+     do {
        pid = waitpid (-1, &status, WNOHANG|WUNTRACED);
        if (pid > 0 && pid == wait_on_pid)
 	   done = 1;
@@ -99,6 +99,7 @@ cleanup(int nsockets, struct x_socket *sockets)
 	    free (sockets[i].pathname);
 	}
     }
+    free(sockets);
 }
 
 /*
@@ -141,7 +142,7 @@ recv_conn (int sock, kx_context *kc,
      }
      kc->thataddr_len = addrlen;
 
-     getnameinfo_verified (kc->thataddr, 
+     getnameinfo_verified (kc->thataddr,
 			   kc->thataddr_len,
 			   remotehost, sizeof(remotehost),
 			   NULL, 0, 0);
@@ -154,10 +155,6 @@ recv_conn (int sock, kx_context *kc,
 #ifdef KRB5
      if (ret && recv_v5_auth (kc, sock, msg) == 0)
 	 ret = 0;
-#endif
-#ifdef KRB4
-     if (ret && recv_v4_auth (kc, sock, msg) == 0)
-	 ret = 0;
 #endif
      if (ret) {
 	 syslog (LOG_ERR, "unrecognized auth protocol: %x %x %x %x",
@@ -174,11 +171,17 @@ recv_conn (int sock, kx_context *kc,
      if (*p != INIT)
 	 fatal(kc, sock, "Bad message");
      p++;
+     if ((p - msg) < sizeof(msg))
+	 fatal(kc, sock, "user");
+
      p += kx_get_int (p, &tmp32, 4, 0);
-     len = min(sizeof(user), tmp32);
-     memcpy (user, p, len);
+     if (tmp32 >= sizeof(user) - 1)
+	 fatal(kc, sock, "user name too long");
+     if ((p - msg) + tmp32 >= sizeof(msg))
+	 fatal(kc, sock, "user too long");
+     memcpy (user, p, tmp32);
      p += tmp32;
-     user[len] = '\0';
+     user[tmp32] = '\0';
 
      passwd = k_getpwnam (user);
      if (passwd == NULL)
@@ -188,6 +191,9 @@ recv_conn (int sock, kx_context *kc,
 	 fatal (kc, sock, "%s not allowed to login as %s",
 		kc->user, user);
 
+     if ((p - msg) >= sizeof(msg))
+	 fatal(kc, sock, "user too long");
+
      flags = *p++;
 
      if (flags & PASSIVE) {
@@ -232,7 +238,7 @@ recv_conn (int sock, kx_context *kc,
      }
 
      ret = getnameinfo(kc->thataddr, kc->thataddr_len,
-		       remoteaddr, sizeof(remoteaddr), 
+		       remoteaddr, sizeof(remoteaddr),
 		       NULL, 0, NI_NUMERICHOST);
      if (ret != 0)
 	 fatal (kc, sock, "getnameinfo failed: %s", gai_strerror(ret));
@@ -243,15 +249,17 @@ recv_conn (int sock, kx_context *kc,
      umask(077);
      if (!(flags & PASSIVE)) {
 	 p += kx_get_int (p, &tmp32, 4, 0);
-	 len = min(tmp32, display_size);
-	 memcpy (display, p, len);
-	 display[len] = '\0';
+	 if (tmp32 > display_size)
+	     fatal(kc, sock, "display too large");
+	 if ((p - msg) + tmp32 + 8 >= sizeof(msg))
+	     fatal(kc, sock, "user too long");
+	 memcpy (display, p, tmp32);
+	 display[tmp32] = '\0';
 	 p += tmp32;
 	 p += kx_get_int (p, &tmp32, 4, 0);
 	 len = min(tmp32, xauthfile_size);
 	 memcpy (xauthfile, p, len);
 	 xauthfile[len] = '\0';
-	 p += tmp32;
      }
 #if defined(SO_KEEPALIVE) && defined(HAVE_SETSOCKOPT)
      if (flags & KEEP_ALIVE) {
@@ -399,7 +407,7 @@ close_connection(int fd, const char *message)
     mlen = strlen(message);
     if(mlen > 255)
 	mlen = 255;
-    
+
     /* read first part of connection packet, to get byte order */
     if(read(fd, buf, 6) != 6) {
 	close(fd);
@@ -418,7 +426,7 @@ close_connection(int fd, const char *message)
 	*p++ = 0;
 	
     /* now fill in length of additional data */
-    if(lsb) { 
+    if(lsb) {
 	buf[6] = (p - buf - 8) / 4;
 	buf[7] = 0;
     }else{
@@ -454,7 +462,7 @@ doit_passive (kx_context *kc,
 	snprintf (display, display_size, "localhost:%u", display_num);
     else
 	snprintf (display, display_size, ":%u", display_num);
-    error = create_and_write_cookie (xauthfile, xauthfile_size, 
+    error = create_and_write_cookie (xauthfile, xauthfile_size,
 				     cookie, cookie_len);
     if (error) {
 	cleanup(nsockets, sockets);
@@ -494,7 +502,7 @@ doit_passive (kx_context *kc,
     memcpy (p, xauthfile, len);
     p += len;
     rem -= len;
-	  
+	
     if(kx_write (kc, sock, msg, p - msg) < 0) {
 	syslog (LOG_ERR, "write: %m");
 	cleanup(nsockets, sockets);
@@ -507,7 +515,7 @@ doit_passive (kx_context *kc,
 	int i;
 	int ret;
 	int cookiesp = TRUE;
-	       
+	
 	FD_ZERO(&fds);
 	if (sock >= FD_SETSIZE) {
 	    syslog (LOG_ERR, "fd too large");
@@ -632,7 +640,7 @@ doit_active (kx_context *kc,
 
     p = msg;
     *p++ = ACK;
-	  
+	
     if(kx_write (kc, sock, msg, p - msg) < 0) {
 	syslog (LOG_ERR, "write: %m");
 	return 1;
@@ -640,7 +648,7 @@ doit_active (kx_context *kc,
     for (;;) {
 	pid_t child;
 	int len;
-	      
+	
 	len = kx_read (kc, sock, msg, sizeof(msg));
 	if (len < 0) {
 	    syslog (LOG_ERR, "read: %m");
@@ -758,10 +766,8 @@ main (int argc, char **argv)
     } else {
 #if defined(KRB5)
 	port = krb5_getportbyname(NULL, "kx", "tcp", KX_PORT);
-#elif defined(KRB4)
-	port = k_getportbyname ("kx", "tcp", htons(KX_PORT));
 #else
-#error define KRB4 or KRB5
+#error define KRB5
 #endif
     }
 
