@@ -1,40 +1,40 @@
 /*
- * Copyright (c) 1997 - 2004 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 1997 - 2005 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "kadmin_locl.h"
 #include <krb5-private.h>
 
-RCSID("$Id: server.c,v 1.39 2004/05/13 17:46:20 joda Exp $");
+RCSID("$Id$");
 
 static kadm5_ret_t
 kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
@@ -47,17 +47,19 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
     char *op = "";
     krb5_principal princ, princ2;
     kadm5_principal_ent_rec ent;
-    char *password, *exp;
+    char *password, *expression;
     krb5_keyblock *new_keys;
     int n_keys;
     char **princs;
     int n_princs;
     krb5_storage *sp;
-    
-    krb5_unparse_name_fixed(context->context, context->caller, 
+
+    krb5_unparse_name_fixed(context->context, context->caller,
 			    client, sizeof(client));
-    
+
     sp = krb5_storage_from_data(in);
+    if (sp == NULL)
+	krb5_errx(context->context, 1, "out of memory");
 
     krb5_ret_int32(sp, &cmd);
     switch(cmd){
@@ -71,6 +73,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	    krb5_free_principal(context->context, princ);
 	    goto fail;
 	}
+	mask |= KADM5_PRINCIPAL;
 	krb5_unparse_name_fixed(context->context, princ, name, sizeof(name));
 	krb5_warnx(context->context, "%s: %s %s", client, op, name);
 	ret = _kadm5_acl_check_permission(context, KADM5_PRIV_GET, princ);
@@ -123,7 +126,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	    kadm5_free_principal_ent(context->context, &ent);
 	    goto fail;
 	}
-	krb5_unparse_name_fixed(context->context, ent.principal, 
+	krb5_unparse_name_fixed(context->context, ent.principal,
 				name, sizeof(name));
 	krb5_warnx(context->context, "%s: %s %s", client, op, name);
 	ret = _kadm5_acl_check_permission(context, KADM5_PRIV_ADD,
@@ -134,7 +137,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	    free(password);
 	    goto fail;
 	}
-	ret = kadm5_create_principal(kadm_handle, &ent, 
+	ret = kadm5_create_principal(kadm_handle, &ent,
 				     mask, password);
 	kadm5_free_principal_ent(kadm_handle, &ent);
 	memset(password, 0, strlen(password));
@@ -154,7 +157,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	    kadm5_free_principal_ent(context, &ent);
 	    goto fail;
 	}
-	krb5_unparse_name_fixed(context->context, ent.principal, 
+	krb5_unparse_name_fixed(context->context, ent.principal,
 				name, sizeof(name));
 	krb5_warnx(context->context, "%s: %s %s", client, op, name);
 	ret = _kadm5_acl_check_permission(context, KADM5_PRIV_MODIFY,
@@ -182,16 +185,17 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	}
 	krb5_unparse_name_fixed(context->context, princ, name, sizeof(name));
 	krb5_unparse_name_fixed(context->context, princ2, name2, sizeof(name2));
-	krb5_warnx(context->context, "%s: %s %s -> %s", 
+	krb5_warnx(context->context, "%s: %s %s -> %s",
 		   client, op, name, name2);
-	ret = _kadm5_acl_check_permission(context, 
+	ret = _kadm5_acl_check_permission(context,
 					  KADM5_PRIV_ADD,
 					  princ2)
-	    || _kadm5_acl_check_permission(context, 
+	    || _kadm5_acl_check_permission(context,
 					   KADM5_PRIV_DELETE,
 					   princ);
 	if(ret){
 	    krb5_free_principal(context->context, princ);
+	    krb5_free_principal(context->context, princ2);
 	    goto fail;
 	}
 	ret = kadm5_rename_principal(kadm_handle, princ, princ2);
@@ -281,7 +285,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	}
 
 	key_data = malloc (n_key_data * sizeof(*key_data));
-	if (key_data == NULL) {
+	if (key_data == NULL && n_key_data != 0) {
 	    ret = ENOMEM;
 	    krb5_free_principal(context->context, princ);
 	    goto fail;
@@ -353,7 +357,7 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	    krb5_free_principal(context->context, princ);
 	    goto fail;
 	}
-	ret = kadm5_randkey_principal(kadm_handle, princ, 
+	ret = kadm5_randkey_principal(kadm_handle, princ,
 				      &new_keys, &n_keys);
 	krb5_free_principal(context->context, princ);
 	krb5_storage_free(sp);
@@ -366,16 +370,18 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 		krb5_store_keyblock(sp, new_keys[i]);
 		krb5_free_keyblock_contents(context->context, &new_keys[i]);
 	    }
+	    free(new_keys);
 	}
 	break;
     }
     case kadm_get_privs:{
-	ret = kadm5_get_privs(kadm_handle, &mask);
+	uint32_t privs;
+	ret = kadm5_get_privs(kadm_handle, &privs);
 	krb5_storage_free(sp);
 	sp = krb5_storage_emem();
 	krb5_store_int32(sp, ret);
 	if(ret == 0)
-	    krb5_store_int32(sp, mask);
+	    krb5_store_uint32(sp, privs);
 	break;
     }
     case kadm_get_princs:{
@@ -384,19 +390,20 @@ kadmind_dispatch(void *kadm_handle, krb5_boolean initial,
 	if(ret)
 	    goto fail;
 	if(tmp){
-	    ret = krb5_ret_string(sp, &exp);
+	    ret = krb5_ret_string(sp, &expression);
 	    if(ret)
 		goto fail;
 	}else
-	    exp = NULL;
-	krb5_warnx(context->context, "%s: %s %s", client, op, exp ? exp : "*");
+	    expression = NULL;
+	krb5_warnx(context->context, "%s: %s %s", client, op,
+		   expression ? expression : "*");
 	ret = _kadm5_acl_check_permission(context, KADM5_PRIV_LIST, NULL);
 	if(ret){
-	    free(exp);
+	    free(expression);
 	    goto fail;
 	}
-	ret = kadm5_get_principals(kadm_handle, exp, &princs, &n_princs);
-	free(exp);
+	ret = kadm5_get_principals(kadm_handle, expression, &princs, &n_princs);
+	free(expression);
 	krb5_storage_free(sp);
 	sp = krb5_storage_emem();
 	krb5_store_int32(sp, ret);
@@ -468,36 +475,23 @@ match_appl_version(const void *data, const char *appl_version)
 
 static void
 handle_v5(krb5_context context,
-	  krb5_auth_context ac,
 	  krb5_keytab keytab,
-	  int len,
 	  int fd)
 {
     krb5_error_code ret;
-    u_char version[sizeof(KRB5_SENDAUTH_VERSION)];
     krb5_ticket *ticket;
     char *server_name;
     char *client;
     void *kadm_handle;
-    ssize_t n;
     krb5_boolean initial;
+    krb5_auth_context ac = NULL;
 
     unsigned kadm_version;
     kadm5_config_params realm_params;
 
-    if (len != sizeof(KRB5_SENDAUTH_VERSION))
-	krb5_errx(context, 1, "bad sendauth len %d", len);
-    n = krb5_net_read(context, &fd, version, len);
-    if (n < 0)
-	krb5_err (context, 1, errno, "reading sendauth version");
-    if (n == 0)
-	krb5_errx (context, 1, "EOF reading sendauth version");
-    if(memcmp(version, KRB5_SENDAUTH_VERSION, len) != 0)
-	krb5_errx(context, 1, "bad sendauth version %.8s", version);
-	
-    ret = krb5_recvauth_match_version(context, &ac, &fd, 
+    ret = krb5_recvauth_match_version(context, &ac, &fd,
 				      match_appl_version, &kadm_version,
-				      NULL, KRB5_RECVAUTH_IGNORE_VERSION, 
+				      NULL, KRB5_RECVAUTH_IGNORE_VERSION,
 				      keytab, &ticket);
     if(ret == KRB5_KT_NOTFOUND)
 	krb5_errx(context, 1, "krb5_recvauth: key not found");
@@ -530,13 +524,13 @@ handle_v5(krb5_context context,
     if (ret)
 	krb5_err (context, 1, ret, "krb5_unparse_name");
     krb5_free_ticket (context, ticket);
-    ret = kadm5_init_with_password_ctx(context, 
-				       client, 
-				       NULL,
-				       KADM5_ADMIN_SERVICE,
-				       &realm_params, 
-				       0, 0, 
-				       &kadm_handle);
+    ret = kadm5_s_init_with_password_ctx(context,
+					 client,
+					 NULL,
+					 KADM5_ADMIN_SERVICE,
+					 &realm_params,
+					 0, 0,
+					 &kadm_handle);
     if(ret)
 	krb5_err (context, 1, ret, "kadm5_init_with_password_ctx");
     v5_loop (context, ac, initial, kadm_handle, fd);
@@ -544,31 +538,37 @@ handle_v5(krb5_context context,
 
 krb5_error_code
 kadmind_loop(krb5_context context,
-	     krb5_auth_context ac,
-	     krb5_keytab keytab, 
+	     krb5_keytab keytab,
 	     int fd)
 {
-    unsigned char tmp[4];
+    u_char buf[sizeof(KRB5_SENDAUTH_VERSION) + 4];
     ssize_t n;
     unsigned long len;
 
-    n = krb5_net_read(context, &fd, tmp, 4);
+    n = krb5_net_read(context, &fd, buf, 4);
     if(n == 0)
 	exit(0);
     if(n < 0)
 	krb5_err(context, 1, errno, "read");
-    _krb5_get_int(tmp, &len, 4);
-    /* this v4 test could probably also go away */
-    if(len > 0xffff && (len & 0xffff) == ('K' << 8) + 'A') {
-	unsigned char v4reply[] = { 
-	    0x00, 0x0c, 
-	    'K', 'Y', 'O', 'U', 'L', 'O', 'S', 'E', 
-	    0x95, 0xb7, 0xa7, 0x08 /* KADM_BAD_VER */
-	};
-	krb5_net_write(context, &fd, v4reply, sizeof(v4reply));
-	krb5_errx(context, 1, "packet appears to be version 4");
-    } else {
-	handle_v5(context, ac, keytab, len, fd);
-    }
+    _krb5_get_int(buf, &len, 4);
+
+    if (len == sizeof(KRB5_SENDAUTH_VERSION)) {
+
+	n = krb5_net_read(context, &fd, buf + 4, len);
+	if (n < 0)
+	    krb5_err (context, 1, errno, "reading sendauth version");
+	if (n == 0)
+	    krb5_errx (context, 1, "EOF reading sendauth version");
+
+	if(memcmp(buf + 4, KRB5_SENDAUTH_VERSION, len) == 0) {
+	    handle_v5(context, keytab, fd);
+	    return 0;
+	}
+	len += 4;
+    } else
+	len = 4;
+
+    handle_mit(context, buf, len, fd);
+
     return 0;
 }
