@@ -32,11 +32,11 @@
 
 #include "kcm_locl.h"
 
-RCSID("$Id: cache.c 14566 2005-02-06 01:22:49Z lukeh $");
+RCSID("$Id$");
 
 static HEIMDAL_MUTEX ccache_mutex = HEIMDAL_MUTEX_INITIALIZER;
 static kcm_ccache_data *ccache_head = NULL;
-static unsigned int ccache_nextid = 0; 
+static unsigned int ccache_nextid = 0;
 
 char *kcm_ccache_nextid(pid_t pid, uid_t uid, gid_t gid)
 {
@@ -47,7 +47,7 @@ char *kcm_ccache_nextid(pid_t pid, uid_t uid, gid_t gid)
     n = ++ccache_nextid;
     HEIMDAL_MUTEX_unlock(&ccache_mutex);
 
-    asprintf(&name, "%d:%u", uid, n);
+    asprintf(&name, "%ld:%u", (long)uid, n);
 
     return name;
 }
@@ -182,27 +182,17 @@ kcm_ccache_alloc(krb5_context context,
 	goto out;
 
     /*
-     * Then try and find an empty slot
-     * XXX we need to recycle slots for this to actually do anything
+     * Create an enpty slot for us.
      */
     if (slot == NULL) {
-	for (; p != NULL; p = p->next) {
-	    if ((p->flags & KCM_FLAGS_VALID) == 0) {
-		slot = p;
-		break;
-	    }
-	}
-
+	slot = (kcm_ccache_data *)malloc(sizeof(*slot));
 	if (slot == NULL) {
-	    slot = (kcm_ccache_data *)malloc(sizeof(*slot));
-	    if (slot == NULL) {
-		ret = KRB5_CC_NOMEM;
-		goto out;
-	    }
-	    slot->next = ccache_head;
-	    HEIMDAL_MUTEX_init(&slot->mutex);
-	    new_slot = 1;
+	    ret = KRB5_CC_NOMEM;
+	    goto out;
 	}
+	slot->next = ccache_head;
+	HEIMDAL_MUTEX_init(&slot->mutex);
+	new_slot = 1;
     }
 
     slot->name = strdup(name);
@@ -219,8 +209,6 @@ kcm_ccache_alloc(krb5_context context,
     slot->client = NULL;
     slot->server = NULL;
     slot->creds = NULL;
-    slot->n_cursor = 0;
-    slot->cursors = NULL;
     slot->key.keytab = NULL;
     slot->tkt_life = 0;
     slot->renew_life = 0;
@@ -247,7 +235,6 @@ kcm_ccache_remove_creds_internal(krb5_context context,
 				 kcm_ccache ccache)
 {
     struct kcm_creds *k;
-    struct kcm_cursor *c;
 
     k = ccache->creds;
     while (k != NULL) {
@@ -260,20 +247,6 @@ kcm_ccache_remove_creds_internal(krb5_context context,
     }
     ccache->creds = NULL;
 
-    /* remove anything that would have pointed into the creds too */
-
-    ccache->n_cursor = 0;
-
-    c = ccache->cursors;
-    while (c != NULL) {
-	struct kcm_cursor *old;
-
-	old = c;
-	c = c->next;
-	free(old);
-    }
-    ccache->cursors = NULL;
-
     return 0;
 }
 
@@ -471,7 +444,7 @@ kcm_ccache_destroy_if_empty(krb5_context context,
     krb5_error_code ret;
 
     KCM_ASSERT_VALID(ccache);
-    
+
     if (ccache->creds == NULL) {
 	ret = kcm_ccache_destroy_internal(context, ccache->name);
     } else
@@ -490,7 +463,7 @@ kcm_ccache_store_cred(krb5_context context,
     krb5_creds *tmp;
 
     KCM_ASSERT_VALID(ccache);
-    
+
     HEIMDAL_MUTEX_lock(&ccache->mutex);
     ret = kcm_ccache_store_cred_internal(context, ccache, creds, copy, &tmp);
     HEIMDAL_MUTEX_unlock(&ccache->mutex);
@@ -498,6 +471,22 @@ kcm_ccache_store_cred(krb5_context context,
     return ret;
 }
 
+struct kcm_creds *
+kcm_ccache_find_cred_uuid(krb5_context context,
+			  kcm_ccache ccache,
+			  kcmuuid_t uuid)
+{
+    struct kcm_creds *c;
+
+    for (c = ccache->creds; c != NULL; c = c->next)
+	if (memcmp(c->uuid, uuid, sizeof(c->uuid)) == 0)
+	    return c;
+
+    return NULL;
+}
+
+
+
 krb5_error_code
 kcm_ccache_store_cred_internal(krb5_context context,
 			       kcm_ccache ccache,
@@ -511,10 +500,11 @@ kcm_ccache_store_cred_internal(krb5_context context,
     for (c = &ccache->creds; *c != NULL; c = &(*c)->next)
 	;
 
-    *c = (struct kcm_creds *)malloc(sizeof(struct kcm_creds));
-    if (*c == NULL) {
+    *c = (struct kcm_creds *)calloc(1, sizeof(**c));
+    if (*c == NULL)
 	return KRB5_CC_NOMEM;
-    }
+
+    RAND_bytes((*c)->uuid, sizeof((*c)->uuid));
 
     *credp = &(*c)->cred;
 
@@ -529,25 +519,9 @@ kcm_ccache_store_cred_internal(krb5_context context,
 	ret = 0;
     }
 
-    (*c)->next = NULL;
-
     return ret;
 }
 
-static void
-remove_cred(krb5_context context,
-	    struct kcm_creds **c)
-{
-    struct kcm_creds *cred;
-
-    cred = *c;
-
-    *c = cred->next;
-
-    krb5_free_cred_contents(context, &cred->cred);
-    free(cred);
-}
-
 krb5_error_code
 kcm_ccache_remove_cred_internal(krb5_context context,
 				kcm_ccache ccache,
@@ -561,7 +535,11 @@ kcm_ccache_remove_cred_internal(krb5_context context,
 
     for (c = &ccache->creds; *c != NULL; c = &(*c)->next) {
 	if (krb5_compare_creds(context, whichfields, mcreds, &(*c)->cred)) {
-	    remove_cred(context, c);
+	    struct kcm_creds *cred = *c;
+
+	    *c = cred->next;
+	    krb5_free_cred_contents(context, &cred->cred);
+	    free(cred);
 	    ret = 0;
 	}
     }
@@ -626,7 +604,7 @@ kcm_ccache_retrieve_cred(krb5_context context,
     krb5_error_code ret;
 
     KCM_ASSERT_VALID(ccache);
-    
+
     HEIMDAL_MUTEX_lock(&ccache->mutex);
     ret = kcm_ccache_retrieve_cred_internal(context, ccache,
 					    whichfields, mcreds, credp);
