@@ -1,39 +1,41 @@
 /*
- * Copyright (c) 1997 - 2005 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 1997 - 2005 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
+#define KRB5_DEPRECATED /* uses v4 functions that will die */
+
 #include "hprop.h"
 
-RCSID("$Id: hprop.c 20456 2007-04-19 20:29:42Z lha $");
+RCSID("$Id$");
 
 static int version_flag;
 static int help_flag;
@@ -72,7 +74,7 @@ open_socket(krb5_context context, const char *hostname, const char *port)
 	warnx ("%s: %s", hostname, gai_strerror(error));
 	return -1;
     }
-    
+
     for (a = ai; a != NULL; a = a->ai_next) {
 	int s;
 
@@ -123,7 +125,7 @@ v5_prop(krb5_context context, HDB *db, hdb_entry_ex *entry, void *appdata)
     if(to_stdout)
 	ret = krb5_write_message(context, &pd->sock, &data);
     else
-	ret = krb5_write_priv_message(context, pd->auth_context, 
+	ret = krb5_write_priv_message(context, pd->auth_context,
 				      &pd->sock, &data);
     krb5_data_free(&data);
     return ret;
@@ -157,12 +159,18 @@ v4_prop(void *arg, struct v4_principal *p)
     ent.entry.kvno = p->kvno;
     ent.entry.keys.len = 3;
     ent.entry.keys.val = malloc(ent.entry.keys.len * sizeof(*ent.entry.keys.val));
+    if (ent.entry.keys.val == NULL)
+	krb5_errx(pd->context, ENOMEM, "malloc");
     if(p->mkvno != -1) {
 	ent.entry.keys.val[0].mkvno = malloc (sizeof(*ent.entry.keys.val[0].mkvno));
+	if (ent.entry.keys.val[0].mkvno == NULL)
+	    krb5_errx(pd->context, ENOMEM, "malloc");
 	*(ent.entry.keys.val[0].mkvno) = p->mkvno;
     } else
 	ent.entry.keys.val[0].mkvno = NULL;
     ent.entry.keys.val[0].salt = calloc(1, sizeof(*ent.entry.keys.val[0].salt));
+    if (ent.entry.keys.val[0].salt == NULL)
+	krb5_errx(pd->context, ENOMEM, "calloc");
     ent.entry.keys.val[0].salt->type = KRB5_PADATA_PW_SALT;
     ent.entry.keys.val[0].key.keytype = ETYPE_DES_CBC_MD5;
     krb5_data_alloc(&ent.entry.keys.val[0].key.keyvalue, DES_KEY_SZ);
@@ -203,7 +211,7 @@ v4_prop(void *arg, struct v4_principal *p)
     }
     ent.entry.created_by.time = time(NULL);
     ALLOC(ent.entry.modified_by);
-    ret = krb5_425_conv_principal(pd->context, p->mod_name, p->mod_instance, 
+    ret = krb5_425_conv_principal(pd->context, p->mod_name, p->mod_instance,
 				  v4_realm, &ent.entry.modified_by->principal);
     if(ret){
 	krb5_warn(pd->context, ret, "%s.%s@%s", p->name, p->instance, v4_realm);
@@ -219,9 +227,9 @@ v4_prop(void *arg, struct v4_principal *p)
     ent.entry.flags.postdate = 1;
     ent.entry.flags.client = 1;
     ent.entry.flags.server = 1;
-    
+
     /* special case password changing service */
-    if(strcmp(p->name, "changepw") == 0 && 
+    if(strcmp(p->name, "changepw") == 0 &&
        strcmp(p->instance, "kerberos") == 0) {
 	ent.entry.flags.forwardable = 0;
 	ent.entry.flags.renewable = 0;
@@ -280,7 +288,7 @@ ka_convert(struct prop_data *pd, int fd, struct ka_entry *ent)
        && (flags & KAFNORMAL) == 0) /* remove special entries */
 	return 0;
     memset(&hdb, 0, sizeof(hdb));
-    ret = krb5_425_conv_principal(pd->context, ent->name, ent->instance, 
+    ret = krb5_425_conv_principal(pd->context, ent->name, ent->instance,
 				  v4_realm, &hdb.entry.principal);
     if(ret) {
 	krb5_warn(pd->context, ret,
@@ -290,10 +298,14 @@ ka_convert(struct prop_data *pd, int fd, struct ka_entry *ent)
     }
     hdb.entry.kvno = ntohl(ent->kvno);
     hdb.entry.keys.len = 3;
-    hdb.entry.keys.val = 
+    hdb.entry.keys.val =
 	malloc(hdb.entry.keys.len * sizeof(*hdb.entry.keys.val));
+    if (hdb.entry.keys.val == NULL)
+	krb5_errx(pd->context, ENOMEM, "malloc");
     hdb.entry.keys.val[0].mkvno = NULL;
     hdb.entry.keys.val[0].salt = calloc(1, sizeof(*hdb.entry.keys.val[0].salt));
+    if (hdb.entry.keys.val[0].salt == NULL)
+	krb5_errx(pd->context, ENOMEM, "calloc");
     if (ka_use_null_salt) {
 	hdb.entry.keys.val[0].salt->type = hdb_pw_salt;
 	hdb.entry.keys.val[0].salt->salt.data = NULL;
@@ -301,9 +313,11 @@ ka_convert(struct prop_data *pd, int fd, struct ka_entry *ent)
     } else {
 	hdb.entry.keys.val[0].salt->type = hdb_afs3_salt;
 	hdb.entry.keys.val[0].salt->salt.data = strdup(afs_cell);
+	if (hdb.entry.keys.val[0].salt->salt.data == NULL)
+	    krb5_errx(pd->context, ENOMEM, "strdup");
 	hdb.entry.keys.val[0].salt->salt.length = strlen(afs_cell);
     }
-    
+
     hdb.entry.keys.val[0].key.keytype = ETYPE_DES_CBC_MD5;
     krb5_data_copy(&hdb.entry.keys.val[0].key.keyvalue,
 		   ent->key,
@@ -320,8 +334,8 @@ ka_convert(struct prop_data *pd, int fd, struct ka_entry *ent)
 	ALLOC(hdb.entry.valid_end);
 	*hdb.entry.valid_end = ntohl(ent->valid_end);
     }
-    
-    if (ntohl(ent->pw_change) != NEVERDATE && 
+
+    if (ntohl(ent->pw_change) != NEVERDATE &&
 	ent->pw_expire != 255 &&
 	ent->pw_expire != 0) {
 	ALLOC(hdb.entry.pw_end);
@@ -341,7 +355,7 @@ ka_convert(struct prop_data *pd, int fd, struct ka_entry *ent)
 	ALLOC(hdb.entry.modified_by);
 	read_block(pd->context, fd, ntohl(ent->mod_ptr), &mod, sizeof(mod));
 	
-	krb5_425_conv_principal(pd->context, mod.name, mod.instance, v4_realm, 
+	krb5_425_conv_principal(pd->context, mod.name, mod.instance, v4_realm,
 				&hdb.entry.modified_by->principal);
 	hdb.entry.modified_by->time = ntohl(ent->mod_time);
 	memset(&mod, 0, sizeof(mod));
@@ -378,7 +392,7 @@ ka_dump(struct prop_data *pd, const char *file)
 	krb5_errx(pd->context, 1, "Version mismatch in header: %ld/%ld",
 		  (long)ntohl(header.version1), (long)ntohl(header.version2));
     if(ntohl(header.version1) != 5)
-	krb5_errx(pd->context, 1, "Unknown database version %ld (expected 5)", 
+	krb5_errx(pd->context, 1, "Unknown database version %ld (expected 5)",
 		  (long)ntohl(header.version1));
     for(i = 0; i < ntohl(header.hashsize); i++){
 	int32_t pos = ntohl(header.hash[i]);
@@ -397,13 +411,13 @@ ka_dump(struct prop_data *pd, const char *file)
 struct getargs args[] = {
     { "master-key", 'm', arg_string, &mkeyfile, "v5 master key file", "file" },
     { "database", 'd',	arg_string, &database, "database", "file" },
-    { "source",   0,	arg_string, &source_type, "type of database to read", 
+    { "source",   0,	arg_string, &source_type, "type of database to read",
       "heimdal"
       "|mit-dump"
       "|krb4-dump"
       "|kaserver"
     },
-      
+
     { "v4-realm", 'r',  arg_string, &v4_realm, "v4 realm to use" },
     { "cell",	  'c',  arg_string, &afs_cell, "name of AFS cell" },
     { "kaspecials", 'S', arg_flag,   &kaspecials_flag, "dump KASPECIAL keys"},
@@ -435,14 +449,14 @@ get_creds(krb5_context context, krb5_ccache *cache)
     krb5_get_init_creds_opt *init_opts;
     krb5_preauthtype preauth = KRB5_PADATA_ENC_TIMESTAMP;
     krb5_creds creds;
-    
+
     ret = krb5_kt_register(context, &hdb_kt_ops);
     if(ret) krb5_err(context, 1, ret, "krb5_kt_register");
 
     ret = krb5_kt_resolve(context, ktname, &keytab);
     if(ret) krb5_err(context, 1, ret, "krb5_kt_resolve");
-    
-    ret = krb5_make_principal(context, &client, NULL, 
+
+    ret = krb5_make_principal(context, &client, NULL,
 			      "kadmin", HPROP_NAME, NULL);
     if(ret) krb5_err(context, 1, ret, "krb5_make_principal");
 
@@ -454,12 +468,12 @@ get_creds(krb5_context context, krb5_ccache *cache)
     if(ret) krb5_err(context, 1, ret, "krb5_get_init_creds");
 
     krb5_get_init_creds_opt_free(context, init_opts);
-    
+
     ret = krb5_kt_close(context, keytab);
     if(ret) krb5_err(context, 1, ret, "krb5_kt_close");
-    
-    ret = krb5_cc_gen_new(context, &krb5_mcc_ops, cache);
-    if(ret) krb5_err(context, 1, ret, "krb5_cc_gen_new");
+
+    ret = krb5_cc_new_unique(context, krb5_cc_type_memory, NULL, cache);
+    if(ret) krb5_err(context, 1, ret, "krb5_cc_new_unique");
 
     ret = krb5_cc_initialize(context, *cache, client);
     if(ret) krb5_err(context, 1, ret, "krb5_cc_initialize");
@@ -515,8 +529,7 @@ iterate (krb5_context context,
     case HPROP_KRB4_DUMP:
 	ret = v4_prop_dump(pd, database_name);
 	if(ret)
-	    krb5_warnx(context, "v4_prop_dump: %s", 
-		       krb5_get_err_text(context, ret));
+	    krb5_warn(context, ret, "v4_prop_dump");
 	break;
     case HPROP_KASERVER:
 	ret = ka_dump(pd, database_name);
@@ -526,8 +539,7 @@ iterate (krb5_context context,
     case HPROP_MIT_DUMP:
 	ret = mit_prop_dump(pd, database_name);
 	if (ret)
-	    krb5_warnx(context, "mit_prop_dump: %s",
-		      krb5_get_err_text(context, ret));
+	    krb5_warn(context, ret, "mit_prop_dump");
 	break;
     case HPROP_HEIMDAL:
 	ret = hdb_foreach(context, db, HDB_F_DECRYPT, v5_prop, pd);
@@ -565,7 +577,7 @@ dump_database (krb5_context context, int type,
 
 static int
 propagate_database (krb5_context context, int type,
-		    const char *database_name, 
+		    const char *database_name,
 		    HDB *db, krb5_ccache ccache,
 		    int optidx, int argc, char **argv)
 {
@@ -584,8 +596,8 @@ propagate_database (krb5_context context, int type,
 
 	port = strchr(host, ':');
 	if(port == NULL) {
-	    snprintf(portstr, sizeof(portstr), "%u", 
-		     ntohs(krb5_getportbyname (context, "hprop", "tcp", 
+	    snprintf(portstr, sizeof(portstr), "%u",
+		     ntohs(krb5_getportbyname (context, "hprop", "tcp",
 					       HPROP_PORT)));
 	    port = portstr;
 	} else
@@ -610,11 +622,10 @@ propagate_database (krb5_context context, int type,
         if (local_realm) {
             krb5_realm my_realm;
             krb5_get_default_realm(context,&my_realm);
-
-	    free (*krb5_princ_realm(context, server));
-            krb5_princ_set_realm(context,server,&my_realm);
+            krb5_principal_set_realm(context,server,my_realm);
+	    krb5_xfree(my_realm);
         }
-    
+
 	auth_context = NULL;
 	ret = krb5_sendauth(context,
 			    &auth_context,
@@ -693,7 +704,7 @@ main(int argc, char **argv)
 
     if(help_flag)
 	usage(0);
-    
+
     if(version_flag){
 	print_version(NULL);
 	exit(0);
@@ -721,7 +732,7 @@ main(int argc, char **argv)
 
 
     if(encrypt_flag && decrypt_flag)
-	krb5_errx(context, 1, 
+	krb5_errx(context, 1,
 		  "only one of `--encrypt' and `--decrypt' is meaningful");
 
     if(source_type != NULL) {
@@ -733,7 +744,7 @@ main(int argc, char **argv)
 
     if(!to_stdout)
 	get_creds(context, &ccache);
-    
+
     if(decrypt_flag || encrypt_flag) {
 	ret = hdb_read_master_key(context, mkeyfile, &mkey5);
 	if(ret && ret != ENOENT)
@@ -741,7 +752,7 @@ main(int argc, char **argv)
 	if(ret)
 	    krb5_errx(context, 1, "No master key file found");
     }
-    
+
     if (IS_TYPE_V4(type) && v4_realm == NULL)
 	krb5_errx(context, 1, "Its a Kerberos 4 database "
 		  "but no realm configured");
@@ -750,9 +761,9 @@ main(int argc, char **argv)
     case HPROP_KASERVER:
 	if (database == NULL)
 	    database = DEFAULT_DATABASE;
-	ka_use_null_salt = krb5_config_get_bool_default(context, NULL, FALSE, 
-							"hprop", 
-							"afs_uses_null_salt", 
+	ka_use_null_salt = krb5_config_get_bool_default(context, NULL, FALSE,
+							"hprop",
+							"afs_uses_null_salt",
 							NULL);
 
 	break;
@@ -781,7 +792,7 @@ main(int argc, char **argv)
     if (to_stdout)
 	exit_code = dump_database (context, type, database, db);
     else
-	exit_code = propagate_database (context, type, database, 
+	exit_code = propagate_database (context, type, database,
 					db, ccache, optidx, argc, argv);
 
     if(ccache != NULL)
