@@ -1,7 +1,7 @@
-/*  Copyright 1992 Simmule Turner and Rich Salz.  All rights reserved. 
+/*  Copyright 1992 Simmule Turner and Rich Salz.  All rights reserved.
  *
- *  This software is not subject to any license of the American Telephone 
- *  and Telegraph Company or of the Regents of the University of California. 
+ *  This software is not subject to any license of the American Telephone
+ *  and Telegraph Company or of the Regents of the University of California.
  *
  *  Permission is granted to anyone to use this software for any purpose on
  *  any computer system, and to alter it and redistribute it freely, subject
@@ -25,7 +25,7 @@
 #include <ctype.h>
 #include <errno.h>
 
-RCSID("$Id: editline.c 22898 2008-04-07 18:52:32Z lha $");
+RCSID("$Id$");
 
 /*
 **  Manifest constants.
@@ -246,7 +246,7 @@ TTYinfo(void)
 	if ((term = getenv("TERM")) == NULL)
 	    term = "dumb";
 	if (tgetent(buff, term) >= 0) {
-	    
+	
 	    tmp = tgetstr("le", &bp);
 	    if (tmp != NULL)
 		backspace = strdup(tmp);
@@ -811,15 +811,16 @@ meta(void)
     if ((c = TTYget()) == EOF)
 	return CSeof;
     /* Also include VT-100 arrows. */
-    if (c == '[' || c == 'O')
-	switch (c = TTYget()) {
-	default:	return ring_bell();
+    if (c == '[' || c == 'O') {
+	switch (TTYget()) {
 	case EOF:	return CSeof;
 	case 'A':	return h_prev();
 	case 'B':	return h_next();
 	case 'C':	return fd_char();
 	case 'D':	return bk_char();
 	}
+	return ring_bell();
+    }
 
     if (isdigit(c)) {
 	for (Repeat = c - '0'; (c = TTYget()) != EOF && isdigit(c); )
@@ -1300,7 +1301,7 @@ argify(unsigned char *line, unsigned char ***avp)
 static el_STATUS
 last_argument(void)
 {
-    unsigned char	**av;
+    unsigned char	**av = NULL;
     unsigned char	*p;
     el_STATUS	s;
     int		ac;
@@ -1317,7 +1318,7 @@ last_argument(void)
     else
 	s = ac ? insert_string(av[ac - 1]) : CSstay;
 
-    if (ac)
+    if (av)
 	free(av);
     free(p);
     return s;
