@@ -1,23 +1,23 @@
 /*
- * Copyright (c) 2005 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2005 Kungliga Tekniska Högskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -31,10 +31,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-RCSID("$Id: destest.c 15269 2005-05-29 16:14:38Z lha $");
-#endif
 
 #ifdef KRB5
 #include <krb5-types.h>
@@ -58,10 +55,10 @@ ecb_test(char key[8], char in[8], char out[8])
     DES_key_schedule s;
 
     memcpy(k, key, 8);
-    DES_set_odd_parity(&k);
     memcpy(indata, in, 8);
     memcpy(ansdata, out, 8);
-    DES_set_key(&k, &s);
+    DES_set_odd_parity(&k);
+    DES_set_key_unchecked(&k, &s);
     DES_ecb_encrypt(&indata, &outdata, &s, 1);
     if (memcmp(outdata, ansdata, sizeof(ansdata)) != 0)
 	errx(1, "des: encrypt");
@@ -73,7 +70,7 @@ ecb_test(char key[8], char in[8], char out[8])
 static void
 ebc3_test(char key1[8], char key2[8], char key3[8], char in[8], char out[8])
 {
-    unsigned char k1[8], k2[8], k3[8], 
+    unsigned char k1[8], k2[8], k3[8],
 	indata[8], outdata[8], outdata2[8], ansdata[8];
     DES_key_schedule s1, s2, s3;
 
@@ -82,9 +79,12 @@ ebc3_test(char key1[8], char key2[8], char key3[8], char in[8], char out[8])
     memcpy(k3, key3, 8);
     memcpy(indata, in, 8);
     memcpy(ansdata, out, 8);
-    DES_set_key(&k1, &s1);
-    DES_set_key(&k2, &s2);
-    DES_set_key(&k3, &s3);
+    DES_set_odd_parity(&k1);
+    DES_set_odd_parity(&k2);
+    DES_set_odd_parity(&k3);
+    DES_set_key_unchecked(&k1, &s1);
+    DES_set_key_unchecked(&k2, &s2);
+    DES_set_key_unchecked(&k3, &s3);
     DES_ecb3_encrypt(&indata, &outdata, &s1, &s2, &s3, 1);
     if (memcmp(outdata, ansdata, sizeof(ansdata)) != 0)
 	errx(1, "des3: encrypt");
@@ -96,7 +96,7 @@ ebc3_test(char key1[8], char key2[8], char key3[8], char in[8], char out[8])
 static void
 cbc_test(char key1[8], char iv[8], char in[24], char out[24])
 {
-    unsigned char k1[8], 
+    unsigned char k1[8],
 	indata[24], outdata[24], outdata2[24], ansdata[24];
     DES_key_schedule s1;
     DES_cblock ivdata;
@@ -105,7 +105,8 @@ cbc_test(char key1[8], char iv[8], char in[24], char out[24])
     memcpy(ivdata, iv, 8);
     memcpy(indata, in, 24);
     memcpy(ansdata, out, 24);
-    DES_set_key(&k1, &s1);
+    DES_set_odd_parity(&k1);
+    DES_set_key_unchecked(&k1, &s1);
     DES_cbc_encrypt(indata, outdata, 24, &s1, &ivdata, 1);
     if (memcmp(outdata, ansdata, sizeof(ansdata)) != 0)
 	errx(1, "cbc: encrypt");
@@ -117,7 +118,7 @@ cbc_test(char key1[8], char iv[8], char in[24], char out[24])
 static void
 cfb64_test(char key1[8], char iv[8], char in[23], char out[23])
 {
-    unsigned char k1[8], 
+    unsigned char k1[8],
 	indata[23], outdata[23], outdata2[23], ansdata[23];
     DES_key_schedule s1;
     DES_cblock ivdata;
@@ -126,7 +127,8 @@ cfb64_test(char key1[8], char iv[8], char in[23], char out[23])
     memcpy(k1, key1, 8);
     memcpy(indata, in, 23);
     memcpy(ansdata, out, 23);
-    DES_set_key(&k1, &s1);
+    DES_set_odd_parity(&k1);
+    DES_set_key_unchecked(&k1, &s1);
     num = 0;
     memcpy(ivdata, iv, 8);
     DES_cfb64_encrypt(indata, outdata, 23, &s1, &ivdata, &num, 1);
@@ -140,7 +142,7 @@ cfb64_test(char key1[8], char iv[8], char in[23], char out[23])
 }
 
 static void
-cbc3_test(char key1[8], char key2[8], char key3[8], 
+cbc3_test(char key1[8], char key2[8], char key3[8],
 	  char iv[8], char in[24], char out[24])
 {
     unsigned char k1[8], k2[8], k3[8],
@@ -154,9 +156,12 @@ cbc3_test(char key1[8], char key2[8], char key3[8],
     memcpy(ivdata, iv, 8);
     memcpy(indata, in, 24);
     memcpy(ansdata, out, 24);
-    DES_set_key(&k1, &s1);
-    DES_set_key(&k2, &s2);
-    DES_set_key(&k3, &s3);
+    DES_set_odd_parity(&k1);
+    DES_set_odd_parity(&k2);
+    DES_set_odd_parity(&k3);
+    DES_set_key_unchecked(&k1, &s1);
+    DES_set_key_unchecked(&k2, &s2);
+    DES_set_key_unchecked(&k3, &s3);
     memcpy(&ivec_copy, &ivdata, sizeof(ivec_copy));
     DES_ede3_cbc_encrypt(indata, outdata, 24,
 			 &s1, &s2, &s3, &ivec_copy, 1);
@@ -173,7 +178,7 @@ cbc3_test(char key1[8], char key2[8], char key3[8],
 static void
 pcbc_test(char key1[8], char iv[8], char in[24], char out[24])
 {
-    unsigned char k1[8], 
+    unsigned char k1[8],
 	indata[24], outdata[24], outdata2[24], ansdata[24];
     DES_key_schedule s1;
     DES_cblock ivdata;
@@ -182,7 +187,8 @@ pcbc_test(char key1[8], char iv[8], char in[24], char out[24])
     memcpy(ivdata, iv, 8);
     memcpy(indata, in, 24);
     memcpy(ansdata, out, 24);
-    DES_set_key(&k1, &s1);
+    DES_set_odd_parity(&k1);
+    DES_set_key_unchecked(&k1, &s1);
     DES_pcbc_encrypt(indata, outdata, 24, &s1, &ivdata, 1);
     if (memcmp(outdata, ansdata, sizeof(ansdata)) != 0)
 	errx(1, "pcbc: encrypt");
@@ -192,7 +198,7 @@ pcbc_test(char key1[8], char iv[8], char in[24], char out[24])
 }
 
 static void
-cbc_cksum(char key1[8], char iv[8], char *in, size_t len, 
+cbc_cksum(char key1[8], char iv[8], char *in, size_t len,
 	  uint32_t ret, char out[8])
 {
     unsigned char k1[8], indata[24], ansdata[8];
@@ -204,7 +210,8 @@ cbc_cksum(char key1[8], char iv[8], char *in, size_t len,
     memcpy(ivdata, iv, 8);
     memcpy(indata, in, len);
     memcpy(ansdata, out, 8);
-    DES_set_key(&k1, &s1);
+    DES_set_odd_parity(&k1);
+    DES_set_key_unchecked(&k1, &s1);
     r = DES_cbc_cksum(indata, &outdata, len, &s1, &ivdata);
     if (ret != r)
 	errx(1, "cbc_cksum: cksum error");
