@@ -145,6 +145,11 @@ hdb_entry_get_pkinit_acl (
 	const hdb_entry */*entry*/,
 	const HDB_Ext_PKINIT_acl **/*a*/);
 
+krb5_error_code
+hdb_entry_get_pkinit_cert (
+	const hdb_entry */*entry*/,
+	const HDB_Ext_PKINIT_cert **/*a*/);
+
 krb5_error_code
 hdb_entry_get_pkinit_hash (
 	const hdb_entry */*entry*/,
@@ -237,13 +242,6 @@ hdb_key2principal (
 	krb5_data */*key*/,
 	krb5_principal /*p*/);
 
-krb5_error_code
-hdb_ldap_common (
-	krb5_context /*context*/,
-	HDB ** /*db*/,
-	const char */*search_base*/,
-	const char */*url*/);
-
 krb5_error_code
 hdb_ldap_create (
 	krb5_context /*context*/,
@@ -348,6 +346,12 @@ hdb_set_master_keyfile (
 	HDB */*db*/,
 	const char */*keyfile*/);
 
+krb5_error_code
+hdb_sqlite_create (
+	krb5_context /*context*/,
+	HDB **/*db*/,
+	const char */*argument*/);
+
 krb5_error_code
 hdb_unlock (int /*fd*/);
 
