@@ -30,7 +30,7 @@ void
 _hx509_abort (
 	const char */*fmt*/,
 	...)
-    __attribute__ ((noreturn, format (printf, 1, 2)));
+     __attribute__ ((noreturn, format (printf, 1, 2)));
 
 int
 _hx509_calculate_path (
@@ -120,9 +120,6 @@ _hx509_certs_keys_get (
 	hx509_certs /*certs*/,
 	hx509_private_key **/*keys*/);
 
-hx509_certs
-_hx509_certs_ref (hx509_certs /*certs*/);
-
 int
 _hx509_check_key_usage (
 	hx509_context /*context*/,
@@ -277,23 +274,15 @@ _hx509_make_expr (
 	void */*arg1*/,
 	void */*arg2*/);
 
-int
-_hx509_map_file (
-	const char */*fn*/,
-	void **/*data*/,
-	size_t */*length*/,
-	struct stat */*rsb*/);
-
 int
 _hx509_map_file_os (
 	const char */*fn*/,
-	heim_octet_string */*os*/,
-	struct stat */*rsb*/);
+	heim_octet_string */*os*/);
 
 int
 _hx509_match_keys (
 	hx509_cert /*c*/,
-	hx509_private_key /*private_key*/);
+	hx509_private_key /*key*/);
 
 int
 _hx509_name_cmp (
@@ -323,7 +312,7 @@ _hx509_name_modify (
 int
 _hx509_parse_private_key (
 	hx509_context /*context*/,
-	const heim_oid */*key_oid*/,
+	const AlgorithmIdentifier */*keyai*/,
 	const void */*data*/,
 	size_t /*len*/,
 	hx509_private_key */*private_key*/);
@@ -520,10 +509,11 @@ _hx509_set_cert_attribute (
 	const heim_oid */*oid*/,
 	const heim_octet_string */*attr*/);
 
-void
-_hx509_unmap_file (
-	void */*data*/,
-	size_t /*len*/);
+int
+_hx509_signature_best_before (
+	hx509_context /*context*/,
+	const AlgorithmIdentifier */*alg*/,
+	time_t /*t*/);
 
 void
 _hx509_unmap_file_os (heim_octet_string */*os*/);
@@ -539,7 +529,7 @@ _hx509_verify_get_time (hx509_verify_ctx /*ctx*/);
 int
 _hx509_verify_signature (
 	hx509_context /*context*/,
-	const Certificate */*signer*/,
+	const hx509_cert /*cert*/,
 	const AlgorithmIdentifier */*alg*/,
 	const heim_octet_string */*data*/,
 	const heim_octet_string */*sig*/);
@@ -547,7 +537,7 @@ _hx509_verify_signature (
 int
 _hx509_verify_signature_bitstring (
 	hx509_context /*context*/,
-	const Certificate */*signer*/,
+	const hx509_cert /*signer*/,
 	const AlgorithmIdentifier */*alg*/,
 	const heim_octet_string */*data*/,
 	const heim_bit_string */*sig*/);
