@@ -10,7 +10,7 @@ extern "C" {
 
 #ifndef HX509_LIB
 #if defined(_WIN32)
-#define HX509_LIB_FUNCTION _stdcall __declspec(dllimport)
+#define HX509_LIB_FUNCTION __stdcall __declspec(dllimport)
 #define HX509_LIB_VARIABLE __declspec(dllimport)
 #else
 #define HX509_LIB_FUNCTION
@@ -289,6 +289,13 @@ hx509_certs_end_seq (
 	hx509_certs /*certs*/,
 	hx509_cursor /*cursor*/);
 
+int
+hx509_certs_filter (
+	hx509_context /*context*/,
+	hx509_certs /*certs*/,
+	const hx509_query */*q*/,
+	hx509_certs */*result*/);
+
 int
 hx509_certs_find (
 	hx509_context /*context*/,
@@ -334,6 +341,9 @@ hx509_certs_next_cert (
 	hx509_cursor /*cursor*/,
 	hx509_cert */*cert*/);
 
+hx509_certs
+hx509_certs_ref (hx509_certs /*certs*/);
+
 int
 hx509_certs_start_seq (
 	hx509_context /*context*/,
@@ -356,6 +366,20 @@ hx509_ci_print_names (
 void
 hx509_clear_error_string (hx509_context /*context*/);
 
+int
+hx509_cms_create_signed (
+	hx509_context /*context*/,
+	int /*flags*/,
+	const heim_oid */*eContentType*/,
+	const void */*data*/,
+	size_t /*length*/,
+	const AlgorithmIdentifier */*digest_alg*/,
+	hx509_certs /*certs*/,
+	hx509_peer_info /*peer*/,
+	hx509_certs /*anchors*/,
+	hx509_certs /*pool*/,
+	heim_octet_string */*signed_data*/);
+
 int
 hx509_cms_create_signed_1 (
 	hx509_context /*context*/,
@@ -398,6 +422,7 @@ hx509_cms_unenvelope (
 	const void */*data*/,
 	size_t /*length*/,
 	const heim_octet_string */*encryptedContent*/,
+	time_t /*time_now*/,
 	heim_oid */*contentType*/,
 	heim_octet_string */*content*/);
 
@@ -412,6 +437,7 @@ int
 hx509_cms_verify_signed (
 	hx509_context /*context*/,
 	hx509_verify_ctx /*ctx*/,
+	unsigned int /*flags*/,
 	const void */*data*/,
 	size_t /*length*/,
 	const heim_octet_string */*signedContent*/,
@@ -472,6 +498,9 @@ hx509_crypto_aes128_cbc (void);
 const AlgorithmIdentifier *
 hx509_crypto_aes256_cbc (void);
 
+void
+hx509_crypto_allow_weak (hx509_crypto /*crypto*/);
+
 int
 hx509_crypto_available (
 	hx509_context /*context*/,
@@ -761,6 +790,12 @@ hx509_parse_name (
 	const char */*str*/,
 	hx509_name */*name*/);
 
+int
+hx509_peer_info_add_cms_alg (
+	hx509_context /*context*/,
+	hx509_peer_info /*peer*/,
+	const AlgorithmIdentifier */*val*/);
+
 int
 hx509_peer_info_alloc (
 	hx509_context /*context*/,
@@ -811,6 +846,12 @@ hx509_pem_write (
 	const void */*data*/,
 	size_t /*size*/);
 
+int
+hx509_print_cert (
+	hx509_context /*context*/,
+	hx509_cert /*cert*/,
+	FILE */*out*/);
+
 void
 hx509_print_stdout (
 	void */*ctx*/,
@@ -833,7 +874,7 @@ hx509_query_free (
 int
 hx509_query_match_cmp_func (
 	hx509_query */*q*/,
-	int (*/*func*/)(void *, hx509_cert),
+	int (*/*func*/)(hx509_context, hx509_cert, void *),
 	void */*ctx*/);
 
 int
@@ -925,6 +966,15 @@ hx509_set_error_stringv (
 	const char */*fmt*/,
 	va_list /*ap*/);
 
+const AlgorithmIdentifier *
+hx509_signature_ecPublicKey (void);
+
+const AlgorithmIdentifier *
+hx509_signature_ecdsa_with_sha1 (void);
+
+const AlgorithmIdentifier *
+hx509_signature_ecdsa_with_sha256 (void);
+
 const AlgorithmIdentifier *
 hx509_signature_md2 (void);
 
@@ -1008,6 +1058,11 @@ hx509_verify_attach_revoke (
 	hx509_verify_ctx /*ctx*/,
 	hx509_revoke_ctx /*revoke_ctx*/);
 
+void
+hx509_verify_ctx_f_allow_best_before_signature_algs (
+	hx509_context /*ctx*/,
+	int /*boolean*/);
+
 void
 hx509_verify_ctx_f_allow_default_trustanchors (
 	hx509_verify_ctx /*ctx*/,
