@@ -1,40 +1,39 @@
 /*
- * Copyright (c) 2008 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 2008 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "krb5_locl.h"
-#include "sqlite3.h"
 
-RCSID("$Id$");
+#include <sqlite3.h>
 
 typedef struct krb5_scache {
     char *name;
@@ -61,7 +60,7 @@ typedef struct krb5_scache {
 
 #define SCACHE_DEF_NAME		"Default-cache"
 #define KRB5_SCACHE_DB	"/tmp/krb5scc_%{uid}"
-#define KRB5_SCACHE_NAME	"SDB:"  SCACHE_DEF_NAME ":" KRB5_SCACHE_DB
+#define KRB5_SCACHE_NAME	"SCC:"  SCACHE_DEF_NAME ":" KRB5_SCACHE_DB
 
 #define SCACHE_INVALID_CID	((sqlite_uint64)-1)
 
@@ -188,14 +187,16 @@ trace(void* ptr, const char * str)
 #endif
 
 static krb5_error_code
-prepare_stmt(krb5_context context, sqlite3 *db, sqlite3_stmt **stmt, const char *str)
+prepare_stmt(krb5_context context, sqlite3 *db,
+	     sqlite3_stmt **stmt, const char *str)
 {
     int ret;
 
     ret = sqlite3_prepare_v2(db, str, -1, stmt, NULL);
     if (ret != SQLITE_OK) {
-	krb5_set_error_string(context, "Failed to prepare stmt %s: %s", 
-			      str, sqlite3_errmsg(db));
+	krb5_set_error_message(context, ENOENT,
+			       N_("Failed to prepare stmt %s: %s", ""),
+			       str, sqlite3_errmsg(db));
 	return ENOENT;
     }
     return 0;
@@ -206,11 +207,12 @@ exec_stmt(krb5_context context, sqlite3 *db, const char *str,
 	  krb5_error_code code)
 {
     int ret;
-    
+
     ret = sqlite3_exec(db, str, NULL, NULL, NULL);
     if (ret != SQLITE_OK && code) {
-	krb5_set_error_string(context, "Execute %s: %s", str,
-			      sqlite3_errmsg(db));
+	krb5_set_error_message(context, code,
+			       N_("scache execute %s: %s", ""), str,
+			       sqlite3_errmsg(db));
 	return code;
     }
     return 0;
@@ -229,7 +231,7 @@ default_db(krb5_context context, sqlite3 **db)
     ret = sqlite3_open_v2(name, db, SQLITE_OPEN_READWRITE, NULL);
     free(name);
     if (ret != SQLITE_OK) {
-	krb5_clear_error_string(context);
+	krb5_clear_error_message(context);
 	return ENOENT;
     }
 	
@@ -279,7 +281,7 @@ get_def_name(krb5_context context, char **str)
 out:
     sqlite3_finalize(stmt);
     sqlite3_close(db);
-    krb5_clear_error_string(context);
+    krb5_clear_error_message(context);
     return ENOENT;
 }
 
@@ -316,7 +318,7 @@ scc_alloc(krb5_context context, const char *name)
 	}
     } else {
 	_krb5_expand_default_cc_name(context, KRB5_SCACHE_DB, &s->file);
-	asprintf(&s->name, "unique-%08x", (unsigned int)s);
+	asprintf(&s->name, "unique-%p", s);
     }
     if (s->file == NULL || s->name == NULL) {
 	scc_free(s);
@@ -334,12 +336,14 @@ open_database(krb5_context context, krb5_scache *s, int flags)
     ret = sqlite3_open_v2(s->file, &s->db, SQLITE_OPEN_READWRITE|flags, NULL);
     if (ret) {
 	if (s->db) {
-	    krb5_set_error_string(context, "Error opening scache file %s: %s", 
-				  s->file, sqlite3_errmsg(s->db));
+	    krb5_set_error_message(context, ENOENT,
+				   N_("Error opening scache file %s: %s", ""),
+				   s->file, sqlite3_errmsg(s->db));
 	    sqlite3_close(s->db);
 	    s->db = NULL;
 	} else
-	    krb5_set_error_string(context, "out of memory");
+	    krb5_set_error_message(context, ENOENT,
+				   N_("malloc: out of memory", ""));
 	return ENOENT;
     }
     return 0;
@@ -355,11 +359,12 @@ create_cache(krb5_context context, krb5_scache *s)
 	ret = sqlite3_step(s->icache);
     } while (ret == SQLITE_ROW);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, "Failed to add scache: %d", ret);
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Failed to add scache: %d", ""), ret);
 	return KRB5_CC_IO;
     }
     sqlite3_reset(s->icache);
-    
+
     s->cid = sqlite3_last_insert_rowid(s->db);
 
     return 0;
@@ -376,7 +381,9 @@ make_database(krb5_context context, krb5_scache *s)
 
     ret = open_database(context, s, 0);
     if (ret) {
+	mode_t oldumask = umask(077);
 	ret = open_database(context, s, SQLITE_OPEN_CREATE);
+	umask(oldumask);
 	if (ret) goto out;
 
 	created_file = 1;
@@ -435,7 +442,7 @@ out:
 }
 
 static krb5_error_code
-bind_principal(krb5_context context, 
+bind_principal(krb5_context context,
 	       sqlite3 *db,
 	       sqlite3_stmt *stmt,
 	       int col,
@@ -451,8 +458,9 @@ bind_principal(krb5_context context,
     ret = sqlite3_bind_text(stmt, col, str, -1, free_krb5);
     if (ret != SQLITE_OK) {
 	krb5_xfree(str);
-	krb5_set_error_string(context, "bind principal: %s",
-			      sqlite3_errmsg(db));
+	krb5_set_error_message(context, ENOMEM,
+			       N_("scache bind principal: %s", ""),
+			       sqlite3_errmsg(db));
 	return ENOMEM;
     }
     return 0;
@@ -477,7 +485,8 @@ scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
 
     s = scc_alloc(context, res);
     if (s == NULL) {
-	krb5_set_error_string (context, "malloc: out of memory");
+	krb5_set_error_message(context, KRB5_CC_NOMEM,
+			       N_("malloc: out of memory", ""));
 	return KRB5_CC_NOMEM;
     }
 
@@ -489,7 +498,8 @@ scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
 
     ret = sqlite3_bind_text(s->scache_name, 1, s->name, -1, NULL);
     if (ret != SQLITE_OK) {
-	krb5_set_error_string(context, "bind name: %s", sqlite3_errmsg(s->db));
+	krb5_set_error_message(context, ENOMEM,
+			       "bind name: %s", sqlite3_errmsg(s->db));
 	scc_free(s);
 	return ENOMEM;
     }
@@ -498,9 +508,11 @@ scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
 
 	if (sqlite3_column_type(s->scache_name, 0) != SQLITE_INTEGER) {
 	    sqlite3_reset(s->scache_name);
-	    krb5_set_error_string(context, "Cache name of wrong type "
-				  "for scache %ld", 
+	    krb5_set_error_message(context, KRB5_CC_END,
+				   N_("Cache name of wrong type "
+				      "for scache %ld", ""),
 				  (unsigned long)s->name);
+	    scc_free(s);
 	    return KRB5_CC_END;
 	}
 
@@ -509,7 +521,7 @@ scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
 	s->cid = SCACHE_INVALID_CID;
     }
     sqlite3_reset(s->scache_name);
-    
+
     (*id)->data.data = s;
     (*id)->data.length = sizeof(*s);
 
@@ -524,7 +536,8 @@ scc_gen_new(krb5_context context, krb5_ccache *id)
     s = scc_alloc(context, NULL);
 
     if (s == NULL) {
-	krb5_set_error_string (context, "malloc: out of memory");
+	krb5_set_error_message(context, KRB5_CC_NOMEM,
+			       N_("malloc: out of memory", ""));
 	return KRB5_CC_NOMEM;
     }
 
@@ -560,9 +573,11 @@ scc_initialize(krb5_context context,
 	} while (ret == SQLITE_ROW);
 	sqlite3_reset(s->dcred);
 	if (ret != SQLITE_DONE) {
-	    krb5_set_error_string(context, "Failed to delete old "
-				  "credentials: %s", sqlite3_errmsg(s->db));
 	    ret = KRB5_CC_IO;
+	    krb5_set_error_message(context, ret,
+				   N_("Failed to delete old "
+				      "credentials: %s", ""),
+				   sqlite3_errmsg(s->db));
 	    goto rollback;
 	}
     }
@@ -577,9 +592,10 @@ scc_initialize(krb5_context context,
     } while (ret == SQLITE_ROW);
     sqlite3_reset(s->ucachep);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, "Failed to bind principal to cache %s",
-			      sqlite3_errmsg(s->db));
 	ret = KRB5_CC_IO;
+	krb5_set_error_message(context, ret,
+			       N_("Failed to bind principal to cache %s", ""),
+			       sqlite3_errmsg(s->db));
 	goto rollback;
     }
 
@@ -619,8 +635,9 @@ scc_destroy(krb5_context context,
     } while (ret == SQLITE_ROW);
     sqlite3_reset(s->dcache);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, "Failed to destroy cache %s: %s",
-			      s->name, sqlite3_errmsg(s->db));
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Failed to destroy cache %s: %s", ""),
+			       s->name, sqlite3_errmsg(s->db));
 	return KRB5_CC_IO;
     }
     return 0;
@@ -634,13 +651,15 @@ encode_creds(krb5_context context, krb5_creds *creds, krb5_data *data)
 
     sp = krb5_storage_emem();
     if (sp == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	return ENOMEM;
     }
 
     ret = krb5_store_creds(sp, creds);
     if (ret) {
-	krb5_set_error_string(context, "Failed to store credential");
+	krb5_set_error_message(context, ret,
+			       N_("Failed to store credential in scache", ""));
 	krb5_storage_free(sp);
 	return ret;
     }
@@ -648,7 +667,8 @@ encode_creds(krb5_context context, krb5_creds *creds, krb5_data *data)
     ret = krb5_storage_to_data(sp, data);
     krb5_storage_free(sp);
     if (ret)
-	krb5_set_error_string(context, "Failed to encode credential");
+	krb5_set_error_message(context, ret,
+			       N_("Failed to encode credential in scache", ""));
     return ret;
 }
 
@@ -661,14 +681,16 @@ decode_creds(krb5_context context, const void *data, size_t length,
 
     sp = krb5_storage_from_readonly_mem(data, length);
     if (sp == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	return ENOMEM;
     }
 
     ret = krb5_ret_creds(sp, creds);
     krb5_storage_free(sp);
     if (ret) {
-	krb5_set_error_string(context, "Failed to read credential");
+	krb5_set_error_message(context, ret,
+			       N_("Failed to read credential in scache", ""));
 	return ret;
     }
     return 0;
@@ -700,7 +722,7 @@ scc_store_cred(krb5_context context,
 	Ticket t;
 	size_t len;
 
-	ret = decode_Ticket(creds->ticket.data, 
+	ret = decode_Ticket(creds->ticket.data,
 			    creds->ticket.length, &t, &len);
 	if (ret == 0) {
 	    if(t.enc_part.kvno)
@@ -727,13 +749,15 @@ scc_store_cred(krb5_context context,
     } while (ret == SQLITE_ROW);
     sqlite3_reset(s->icred);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, "Failed to add credential: %s", 
-			      sqlite3_errmsg(s->db));
+	ret = KRB5_CC_IO;
+	krb5_set_error_message(context, ret,
+			       N_("Failed to add credential: %s", ""),
+			       sqlite3_errmsg(s->db));
 	goto rollback;
     }
 
     credid = sqlite3_last_insert_rowid(s->db);
-    
+
     {
 	bind_principal(context, s->db, s->iprincipal, 1, creds->server);
 	sqlite3_bind_int(s->iprincipal, 2, 1);
@@ -744,8 +768,10 @@ scc_store_cred(krb5_context context,
 	} while (ret == SQLITE_ROW);
 	sqlite3_reset(s->iprincipal);
 	if (ret != SQLITE_DONE) {
-	    krb5_set_error_string(context, "Failed to add principal: %s", 
-				  sqlite3_errmsg(s->db));
+	    ret = KRB5_CC_IO;
+	    krb5_set_error_message(context, ret,
+				   N_("Failed to add principal: %s", ""),
+				   sqlite3_errmsg(s->db));
 	    goto rollback;
 	}
     }
@@ -760,8 +786,10 @@ scc_store_cred(krb5_context context,
 	} while (ret == SQLITE_ROW);
 	sqlite3_reset(s->iprincipal);
 	if (ret != SQLITE_DONE) {
-	    krb5_set_error_string(context, "Failed to add principal: %s", 
-				  sqlite3_errmsg(s->db));
+	    ret = KRB5_CC_IO;
+	    krb5_set_error_message(context, ret,
+				   N_("Failed to add principal: %s", ""),
+				   sqlite3_errmsg(s->db));
 	    goto rollback;
 	}
     }
@@ -772,8 +800,6 @@ scc_store_cred(krb5_context context,
     return 0;
 
 rollback:
-    krb5_set_error_string(context, "store credentials: %s",
-			  sqlite3_errmsg(s->db));
     exec_stmt(context, s->db, "ROLLBACK", 0);
 
     return ret;
@@ -798,25 +824,27 @@ scc_get_principal(krb5_context context,
 
     if (sqlite3_step(s->scache) != SQLITE_ROW) {
 	sqlite3_reset(s->scache);
-	krb5_set_error_string(context, "No principal for cache SCACHE:%s:%s", 
-			      s->name, s->file);
+	krb5_set_error_message(context, KRB5_CC_END,
+			       N_("No principal for cache SCC:%s:%s", ""),
+			       s->name, s->file);
 	return KRB5_CC_END;
     }
 	
     if (sqlite3_column_type(s->scache, 0) != SQLITE_TEXT) {
 	sqlite3_reset(s->scache);
-	krb5_set_error_string(context, "Principal data of wrong type "
-			      "for SCACHE:%s:%s", 
-			      s->name, s->file);
+	krb5_set_error_message(context, KRB5_CC_END,
+			       N_("Principal data of wrong type "
+				  "for SCC:%s:%s", ""),
+			       s->name, s->file);
 	return KRB5_CC_END;
     }
 
     str = (const char *)sqlite3_column_text(s->scache, 0);
     if (str == NULL) {
 	sqlite3_reset(s->scache);
-	krb5_set_error_string(context, "Principal not set "
-			      "for SCACHE:%s:%s", 
-			      s->name, s->file);
+	krb5_set_error_message(context, KRB5_CC_END,
+			       N_("Principal not set for SCC:%s:%s", ""),
+			       s->name, s->file);
 	return KRB5_CC_END;
     }
 
@@ -847,7 +875,8 @@ scc_get_first (krb5_context context,
 
     ctx = calloc(1, sizeof(*ctx));
     if (ctx == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	return ENOMEM;
     }
 
@@ -858,8 +887,9 @@ scc_get_first (krb5_context context,
     }
 
     if (s->cid == SCACHE_INVALID_CID) {
-	krb5_set_error_string(context, "Iterating a invalid cache %s", 
-			      s->name);
+	krb5_set_error_message(context, KRB5_CC_END,
+			       N_("Iterating a invalid scache %s", ""),
+			       s->name);
 	free(ctx);
 	return KRB5_CC_END;
     }
@@ -867,21 +897,23 @@ scc_get_first (krb5_context context,
     asprintf(&name, "credIteration%luPid%d",
 	     (unsigned long)ctx, (int)getpid());
     if (name == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	free(ctx);
 	return ENOMEM;
     }
 
     asprintf(&ctx->drop, "DROP TABLE %s", name);
     if (ctx->drop == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	free(name);
 	free(ctx);
 	return ENOMEM;
     }
 
     asprintf(&str, "CREATE TEMPORARY TABLE %s "
-	     "AS SELECT oid,created_at FROM credentials WHERE cid = %lu", 
+	     "AS SELECT oid,created_at FROM credentials WHERE cid = %lu",
 	     name, (unsigned long)s->cid);
 
     ret = exec_stmt(context, s->db, str, KRB5_CC_IO);
@@ -912,7 +944,7 @@ scc_get_first (krb5_context context,
 	return ret;
     }
 
-    ret = prepare_stmt(context, s->db, &ctx->credstmt, 
+    ret = prepare_stmt(context, s->db, &ctx->credstmt,
 		       "SELECT cred FROM credentials WHERE oid = ?");
     if (ret) {
 	sqlite3_finalize(ctx->stmt);
@@ -943,11 +975,12 @@ scc_get_next (krb5_context context,
 next:
     ret = sqlite3_step(ctx->stmt);
     if (ret == SQLITE_DONE) {
-	krb5_clear_error_string(context);
+	krb5_clear_error_message(context);
         return KRB5_CC_END;
     } else if (ret != SQLITE_ROW) {
-	krb5_set_error_string(context, "Database failed: %s", 
-			      sqlite3_errmsg(s->db));
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("scache Database failed: %s", ""),
+			       sqlite3_errmsg(s->db));
         return KRB5_CC_IO;
     }
 
@@ -964,9 +997,9 @@ next:
     }
 
     if (sqlite3_column_type(ctx->credstmt, 0) != SQLITE_BLOB) {
-	krb5_set_error_string(context, "credential of wrong type "
-			      "for SCACHE:%s:%s", 
-			      s->name, s->file);
+	krb5_set_error_message(context, KRB5_CC_END,
+			       N_("credential of wrong type for SCC:%s:%s", ""),
+			       s->name, s->file);
 	sqlite3_reset(ctx->credstmt);
 	return KRB5_CC_END;
     }
@@ -976,8 +1009,6 @@ next:
 
     ret = decode_creds(context, data, len, creds);
     sqlite3_reset(ctx->credstmt);
-
-    krb5_clear_error_string(context);
     return ret;
 }
 
@@ -988,12 +1019,11 @@ scc_end_get (krb5_context context,
 {
     struct cred_ctx *ctx = *cursor;
     krb5_scache *s = SCACHE(id);
-    int ret;
 
     sqlite3_finalize(ctx->stmt);
     sqlite3_finalize(ctx->credstmt);
 
-    ret = exec_stmt(context, s->db, ctx->drop, 0);
+    exec_stmt(context, s->db, ctx->drop, 0);
 
     free(ctx->drop);
     free(ctx);
@@ -1018,7 +1048,7 @@ scc_remove_cred(krb5_context context,
     if (ret)
 	return ret;
 
-    ret = prepare_stmt(context, s->db, &stmt, 
+    ret = prepare_stmt(context, s->db, &stmt,
 		       "SELECT cred,oid FROM credentials "
 		       "WHERE cid = ?");
     if (ret)
@@ -1035,17 +1065,19 @@ scc_remove_cred(krb5_context context,
 	    ret = 0;
 	    break;
 	} else if (ret != SQLITE_ROW) {
-	    krb5_set_error_string(context, "Database failed: %s", 
-				  sqlite3_errmsg(s->db));
 	    ret = KRB5_CC_IO;
+	    krb5_set_error_message(context, ret,
+				   N_("scache Database failed: %s", ""),
+				   sqlite3_errmsg(s->db));
 	    break;
 	}
 
 	if (sqlite3_column_type(stmt, 0) != SQLITE_BLOB) {
-	    krb5_set_error_string(context, "credential of wrong type "
-				  "for SCACHE:%s:%s", 
-				  s->name, s->file);
 	    ret = KRB5_CC_END;
+	    krb5_set_error_message(context, ret,
+				   N_("Credential of wrong type "
+				      "for SCC:%s:%s", ""),
+				   s->name, s->file);
 	    break;
 	}
 
@@ -1053,10 +1085,8 @@ scc_remove_cred(krb5_context context,
 	len = sqlite3_column_bytes(stmt, 0);
 
 	ret = decode_creds(context, data, len, &creds);
-	if (ret) {
-	    krb5_set_error_string(context, "failed to decode creds");
+	if (ret)
 	    break;
-	}
 	
 	ret = krb5_compare_creds(context, which, mcreds, &creds);
 	krb5_free_cred_contents(context, &creds);
@@ -1070,7 +1100,7 @@ scc_remove_cred(krb5_context context,
     sqlite3_finalize(stmt);
 
     if (id) {
-	ret = prepare_stmt(context, s->db, &stmt, 
+	ret = prepare_stmt(context, s->db, &stmt,
 			   "DELETE FROM credentials WHERE oid=?");
 	if (ret)
 	    return ret;
@@ -1081,8 +1111,9 @@ scc_remove_cred(krb5_context context,
 	} while (ret == SQLITE_ROW);
 	sqlite3_finalize(stmt);
 	if (ret != SQLITE_DONE) {
-	    krb5_set_error_string(context, "failed to delete credentail");
 	    ret = KRB5_CC_IO;
+	    krb5_set_error_message(context, ret,
+				   N_("failed to delete scache credental", ""));
 	} else
 	    ret = 0;
     }
@@ -1097,7 +1128,7 @@ scc_set_flags(krb5_context context,
 {
     return 0; /* XXX */
 }
-		    
+		
 struct cache_iter {
     char *drop;
     sqlite3 *db;
@@ -1115,7 +1146,8 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 
     ctx = calloc(1, sizeof(*ctx));
     if (ctx == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	return ENOMEM;
     }
 
@@ -1128,7 +1160,8 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
     asprintf(&name, "cacheIteration%luPid%d",
 	     (unsigned long)ctx, (int)getpid());
     if (name == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
 	free(ctx);
 	return ENOMEM;
@@ -1136,17 +1169,19 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 
     asprintf(&ctx->drop, "DROP TABLE %s", name);
     if (ctx->drop == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
 	free(name);
 	free(ctx);
 	return ENOMEM;
     }
 
-    asprintf(&str, "CREATE TEMPORARY TABLE %s AS SELECT name FROM caches", 
+    asprintf(&str, "CREATE TEMPORARY TABLE %s AS SELECT name FROM caches",
 	     name);
     if (str == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
 	free(name);
 	free(ctx->drop);
@@ -1174,8 +1209,9 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 	free(ctx);
 	return ENOMEM;
     }
-    
+
     ret = prepare_stmt(context, ctx->db, &ctx->stmt, str);
+    free(str);
     if (ret) {
 	exec_stmt(context, ctx->db, ctx->drop, 0);
 	sqlite3_close(ctx->db);
@@ -1197,16 +1233,16 @@ scc_get_cache_next(krb5_context context,
     struct cache_iter *ctx = cursor;
     krb5_error_code ret;
     const char *name;
-    krb5_scache *s = NULL;
 
 again:
     ret = sqlite3_step(ctx->stmt);
     if (ret == SQLITE_DONE) {
-	krb5_clear_error_string(context);
+	krb5_clear_error_message(context);
         return KRB5_CC_END;
     } else if (ret != SQLITE_ROW) {
-	krb5_set_error_string(context, "Database failed: %s", 
-			      sqlite3_errmsg(s->db));
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Database failed: %s", ""),
+			       sqlite3_errmsg(ctx->db));
         return KRB5_CC_IO;
     }
 
@@ -1245,9 +1281,10 @@ scc_move(krb5_context context, krb5_ccache from, krb5_ccache to)
     krb5_error_code ret;
 
     if (strcmp(sfrom->file, sto->file) != 0) {
-	krb5_set_error_string(context, "Can't handle cross database "
-			      "credential move: %s -> %s", 
-			      sfrom->file, sto->file);
+	krb5_set_error_message(context, KRB5_CC_BADNAME,
+			       N_("Can't handle cross database "
+				  "credential move: %s -> %s", ""),
+			       sfrom->file, sto->file);
 	return KRB5_CC_BADNAME;
     }
 
@@ -1268,8 +1305,9 @@ scc_move(krb5_context context, krb5_ccache from, krb5_ccache to)
 	} while (ret == SQLITE_ROW);
 	sqlite3_reset(sfrom->dcache);
 	if (ret != SQLITE_DONE) {
-	    krb5_set_error_string(context, 
-				  "Failed to delete old cache: %d", (int)ret);
+	    krb5_set_error_message(context, KRB5_CC_IO,
+				   N_("Failed to delete old cache: %d", ""),
+				   (int)ret);
 	    goto rollback;
 	}
     }
@@ -1282,8 +1320,9 @@ scc_move(krb5_context context, krb5_ccache from, krb5_ccache to)
     } while (ret == SQLITE_ROW);
     sqlite3_reset(sfrom->ucachen);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, 
-			      "Failed to update new cache: %d", (int)ret);
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Failed to update new cache: %d", ""),
+			       (int)ret);
 	goto rollback;
     }
 
@@ -1292,13 +1331,13 @@ scc_move(krb5_context context, krb5_ccache from, krb5_ccache to)
     ret = exec_stmt(context, sfrom->db, "COMMIT", KRB5_CC_IO);
     if (ret) return ret;
 
-    /* free sfrom */
+    scc_free(sfrom);
 
     return 0;
 
 rollback:
-
     exec_stmt(context, sfrom->db, "ROLLBACK", 0);
+    scc_free(sfrom);
 
     return KRB5_CC_IO;
 }
@@ -1313,10 +1352,11 @@ scc_get_default_name(krb5_context context, char **str)
     if (ret)
 	return _krb5_expand_default_cc_name(context, KRB5_SCACHE_NAME, str);
 
-    asprintf(str, "SDB:%s", name);
+    asprintf(str, "SCC:%s", name);
     free(name);
     if (*str == NULL) {
-	krb5_set_error_string(context, "malloc: out of memory");
+	krb5_set_error_message(context, ENOMEM,
+			       N_("malloc: out of memory", ""));
 	return ENOMEM;
     }
     return 0;
@@ -1329,17 +1369,19 @@ scc_set_default(krb5_context context, krb5_ccache id)
     krb5_error_code ret;
 
     if (s->cid == SCACHE_INVALID_CID) {
-	krb5_set_error_string(context,
-			      "Trying to set a invalid cache as default %s",
-			      s->name);
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Trying to set a invalid cache "
+				  "as default %s", ""),
+			       s->name);
 	return KRB5_CC_IO;
     }
 
     ret = sqlite3_bind_text(s->umaster, 1, s->name, -1, NULL);
     if (ret) {
 	sqlite3_reset(s->umaster);
-	krb5_set_error_string(context, "Failed to set name of default cache");
-	return ret;
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Failed to set name of default cache", ""));
+	return KRB5_CC_IO;
     }
 
     do {
@@ -1347,7 +1389,8 @@ scc_set_default(krb5_context context, krb5_ccache id)
     } while (ret == SQLITE_ROW);
     sqlite3_reset(s->umaster);
     if (ret != SQLITE_DONE) {
-	krb5_set_error_string(context, "Failed to update default cache");
+	krb5_set_error_message(context, KRB5_CC_IO,
+			       N_("Failed to update default cache", ""));
 	return KRB5_CC_IO;
     }
 
@@ -1355,14 +1398,14 @@ scc_set_default(krb5_context context, krb5_ccache id)
 }
 
 /**
- * Variable containing the SDB based credential cache implemention.
+ * Variable containing the SCC based credential cache implemention.
  *
  * @ingroup krb5_ccache
  */
 
 KRB5_LIB_VARIABLE const krb5_cc_ops krb5_scc_ops = {
     KRB5_CC_OPS_VERSION,
-    "SDB",
+    "SCC",
     scc_get_name,
     scc_resolve,
     scc_gen_new,
